.\" 
.\" Copyright (c) 2004 Gordon D. Carrie. All rights reserved.
.\" 
.\" Licensed under the Open Software License version 2.1
.\" 
.\" Please address questions and feedback to user0@tkgeomap.org
.\"
.\" @(#) $Id: igeomap.n,v 1.15 2005/10/28 19:02:57 tkgeomap Exp $
.\"
.so man.macros
.TH geomap::igeomap n 2 Tk "Tk Extensions tkgeomap"
.SH NAME
igeomap \- create and manipulate a geographic map with interactive menus and bindings.
.SH SYNOPSIS
\fBpackage require tkgeomap_procs\fR ?\fB2\fR?
.br
\fBgeomap::igeomap\fR \fInamespaceName\fR \fIpathName\fR ?\fIoptions\fR?
.SH DESCRIPTION
The \fBgeomap::igeomap\fR procedure creates a frame named \fIpathName\fR,
which contains a canvas with a geographic map and associated menus.
It also creates or modifies a namespace named \fInamespaceName\fR.
When the \fBgeomap::igeomap\fR procedure is called, \fIpathName\fR should not
exist, although its parent should.  Namespace \fInamespaceName\fR may already
exist.  If not, it will be created.  The \fBigeomap\fR procedure packs the
contents of frame \fIpathName\fR, but it does not submit it to any geometry
manager.  The application that creates the igeomap should eventually make
the frame visible using Tk geometry management commands, such as \fBpack\fR,
\fBplace\fR, or \fBgrid\fR.
.sp
Namespace procedures described below create and manage \fBgeomap_lnarr\fR and
\fBgeomap_place\fR items in the canvas. These items display
information at geographic locations, as described in the \fBtkgeomap\fR (n)
man page.  Items created with the namespace procedures receive the same
\fB-refpoint\fR, \fB-projection\fR, \fB-scale\fR, and \fB-rotation\fR
configuration options so that the items are on the same map in the canvas, and
they are updated automatically when the map's cartographic properties change.
This is more convenient than configuring each canvas item separately.
There are also procedures that create and manage menus and bindings that make
the map interactive.
.sp
Tasks not done by the namespace procedures can be done with widget commands,
whose names are stored in namespace variables.  \fInamespaceNmae\fB::frame\fR
is the path name of the frame the holds the map canvas and menus, which was
given as \fIpathName\fR in the initial call to \fBgeomap::igeomap\fR.
\fInamespaceName\fB::canvas\fR is the path name of the canvas.
\fInamespaceName\fB::mbar\fR is the path name of the menu bar (a frame).
\fInamespaceName\fB::toplevel\fR is the path name of the toplevel widget that
contains everything.  Array member \fB::geomap::map_ns\fR(\fItoplevel_path\fR)
maps the toplevel widget name to the name of the map namespace.
.sp
The \fBgeomap::igeomap\fR procedure accepts additional \fIoption value\fR
pairs which initialize the map and canvas configuration.  These
\fIoption value\fR pairs are passed to the \fInamespaceName::configure\fR
procedure, described below.
.SH NAMESPACE PROCEDURES
.TP
\fInamespaceName\fB::configure\fR \fIoption value ?option value ...?\fR
This procedure sets namespace variables that define the cartographic properties
of the map and configures associated items in the canvas.  If a given
\fIoption\fR is not in the list below, the procedure applies
it to the canvas configuration.  If \fIoption\fR is not a valid canvas
configuration option either, the procedure raises an error.  The following
options are recognized.
.RS
.TP
\fB-refpoint\fR \fI{lat lon}\fR
Sets the map center to point \fI{lat lon}\fR.  Default value is \fB{0.0 0.0}\fR.
.TP
\fB-projname\fR \fIname\fR
Specifies the type of map projection.  \fIname\fR should identify a projection
from the list returned by the \fBgeomap::projections\fR command.
Unlike the \fBgeomap::projection\fR command, this
option does not require any projection parameters.  If the projection requires
a reference point, such as Orthographic and Lambert Conformal Conic, the
lat-lon at the center of the map is used.  If the projection requires a
reference longitude, such as Mercator, the longitude at the center of the map is
used.  Projections that require a hemisphere specification, such as Polar
Stereographic, use whichever hemisphere contains the map center.
Array ::geomap::ProjRef maps projection names to projection reference, which
is either \fBlongitude\fR or \fBpoint\fR.
The default value for the \fB-projname\fR option is \fBMercator\fR.
.TP
\fB-scale\fR \fIscale\fR
Specifies the cartographic scale of the map.  The value can be a
floating point number or a cartographic scale (e.g. 1:30000000).  Default value
is \fB1.0e-7\fR.
.TP
\fB-rotation\fR \fIrotation\fR
Specifies the map's orientation.  The \fIrotation\fR should be a value known
to the \fB-rotation\fR option described in the \fBtkgeomap\fR (n) man page.
Default value is \fB0.0\fR.
.TP
\fB-update\fR \fIscript\fR
Specifies a script to evaluate whenever the \fB-refpoint\fR,
\fB-projection\fR, \fB-scale\fR, or \fB-rotation\fR option is changed.  This
makes it possible to update labels and legends around the map, add or remove
items as the scale changes, and perform any other tasks that depend on the
map's cartographic characteristics.  \fIscript\fR is evaluated at global scope.
Default value is \fB{}\fR.
.TP
\fB-boundcirclecolor\fR \fIcolor\fR
Specifies the color of the bounding circle for hemispheric projections.
Default value is \fBBlack\fR.
.TP
\fB-lazy\fR \fIboolean\fR
Deletes certain bindings associated with the \fB-projname\fR option.  This
option should be set to true if the widget appears to be wasting time
modifying the map projection without changing the map appearance.
This often happens when the map is zoomed in most of the time.
Value should be a Boolean.  Default value is \fB0\fR (false).
.TP
\fB-colormenu\fR \fIboolean\fR
Determines whether the \fBFile\fR menu will have a \fBColors\fR selection.
This selection creates a dialog in which the user interactively selects colors
for various map elements in the display.  It assumes that map colors are set
with two arrays in the map namespace:
\fBnamespaceName::colorval\fR and \fBnamespaceName::colorscript\fR.
These two arrays use the same set of indeces, which should identify map
elements that share a color, e.g. \fIwater\fR, \fIland\fR, \fIplaces\fR, etc.
Array values in the \fBnamespaceName::colorval\fR array are color values to
use when displaying the map elements.  They should be intelligible to
\fBTk_GetColor\fR.
Array values in the \fBnamespaceName::colorscript\fR array are scripts to run
when the corresponding color changes.  Usually they draw or configure the
affected map elements.
The dialog box will have a button for each map element represented in the
\fBcolorval\fR and \fBcolorscript\fR arrays.  When the user presses one of
these buttons, a color selection dialog will appear.  If the user selects a
color and clicks \fBOK\fR in the color selection dialog, the corresponding
script from the \fBcolorscript\fR array will be called.  This script should
redraw affected items with the chosen color.
The \fBSave\fR button saves color values currently in the dialog to a file
identified by the user.  It adds or modifies a line in the selected file
that starts with "Colors:" followed by the contents of the
\fBnamespaceName::colorval\fR array.
The \fBLoad\fR button retrieves values from a file created with the \fBSave\fR
button.  The \fBUndo\fR button retrieves previous color configurations.
The path name of the color dialog will be stored in the \fBcolor_dlg\fR
variable in the map namespace.
If the \fB-colormenu\fR configuration option is set to true, the application
should use the the \fBcolorval\fR and \fBcolorscript\fR arrays to manage all
colors in the map.  This will allow the user to modify colors conveniently in
one place.  The specific indeces and values in these arrays, and the manner
in which they are set, will depend on what data the map displays and how it
is imported.
.TP
\fB-layermenu\fR \fIboolean\fR
Determines whether the \fBFile\fR menu will have a \fBLayers\fR selection.
This selection creates a dialog in which the user can adjust the stacking order
of certain canvas items by dragging rectangles representing canvas
tags.  Items without tags listed in the dialog move to the bottom of the
stacking order.  Tags can be added to and removed from this dialog with the map
namespace \fBinsert_layer\fR and \fBrm_layer\fR procedures, described below.
The path name of the layer dialog will be stored in the \fBlayer_dlg\fR
variable in the map namespace.
.TP
\fB-projections\fR \fIprojectionList\fR
Lists the projection names in the \fBProjections\fR menu.  Selecting a
projection from the menu sets the \fB-projname\fR option to that projection.
Default value is \fB{CylEqDist Mercator CylEqArea LambertConfConic
LambertEqArea Stereographic PolarStereographic Orthographic}\fR.  If value is
\fB{}\fR, the \fBProjections\fR menu is not displayed.
.TP
\fB-scales\fR \fIscaleList\fR
Lists the scales in the \fBScales\fR menu at the top of the
map.  Selecting a scale from the menu sets the \fB-scale\fR option to that
scale.  Default value is \fB{1:10000000 1:20000000 1:30000000 1:45000000
1:60000000 1:90000000 1:120000000}\fR.  If value is \fB{}\fR, the \fBScales\fR
menu is not displayed.
.TP
\fB-rotationmenu\fR \fIboolean\fR
Specifies whether or not the \fBTop\fR menu should be displayed.  Selecting
a direction from the menu sets the \fB-rotation\fR option so that the chosen
direction is at the top of the map.  Value should be a Boolean.  Default value
is \fB1\fR.
.RE
.TP
\fInamespaceName\fB::cget\fR \fIoption\fR
Retrieves a value from the map configuration.  \fIoption\fR can be one of the
options known to the \fBconfigure\fR procedure, or a canvas option.
.TP
\fInamespaceName\fB::draw\fR \fItype name\fR ?\fIoption value ...\fR?
This procedure creates or modifies a map item in the canvas.  \fItype\fR
identifies the item type, which must be either \fBgeomap_lnarr\fR or
\fBgeomap_place\fR.
\fIname\fR identifies the linearray or place displayed in the item.
It should be the name of a linearray or place command returned by the
\fBgeomap::lnarr\fR or \fBgeomap::place\fR command.  See the \fBtclgeomap\fR
(n) man page for more information on linearrays and places.
If the command for the linearray or place is in a non-global namespace,
\fIname\fR MUST BE FULLY QUALIFIED.
If \fItype\fR and \fIname\fR refer to an already existing item, the item is
modified.  Otherwise, a new item is created.
Item options that define the geographic map, specifically \fB-refpoint\fR,
\fB-projection\fR, \fB-rotation\fR, and \fB-scale\fR, will be obtained from
the namespace.
Other options that affect the item's appearance can be given in the
\fIoption value\fR list.  See \fBtkgeomap\fR (n) man page for discussion of
recognized options.  Any options in the \fIoption value\fR list that define
the geographic map will be ignored.
In addition to any tags specified in the option list, the item will have the
following tags: \fIname\fR, and \fBgeomap\fR.
These tags help identify the item in other procedures and configuration tasks.
This procedure returns the item identifier for the new or modified item.
.TP
\fInamespaceName\fB::erase\fR \fItype name\fR
Deletes an item created by the \fBdraw\fR command.  \fItype\fR and \fIname\fR
should be as given to the \fBdraw\fR command.
.TP
\fInamespaceName\fB::insert_layer\fR \fIlayer\fR ?\fIindex\fR?
Allows the \fBFile->Layers\fR dialog to manipulate the stacking order of map
canvas items with tag \fIlayer\fR.  Layers can be arranged in a hierarchy.
A \fB:\fR in \fIlayer\fR denotes a separate level in the map hierarchy.  Text
to the left of a \fB:\fR identifies a parent.  Text to the right identifies
a child or hierarchy of children.  If several hierarchical levels of layers
are present, the dialog will show them in a tree diagram.  Each path through
the tree is one layer.  All children of an item move with the item.  An item
can only be moved relative to its siblings (other children of its parent).
If present, \fIindex\fR specifies where to put these items in the stacking
order.  \fIindex\fR should specify a list index intelligible to the
\fBlinsert\fR command.  A smaller value means the items are lower in the
stacking order.  Default is \fBend\fR (items with \fIlayer\fR tag go on top).
.TP
\fInamespaceName\fB::rm_layer\fR \fIlayer\fR
Removes tag \fIlayer\fR from the list of layers managed by the
\fBFile->Layers\fR dialog.  Canvas items with tag \fIlayer\fR will still exist,
but the \fBFile->Layers\fR dialog will no longer control their stacking order.
.TP
\fInamespaceName\fB::set_layers\fR
Puts items with layer related tags into their stacking order.  This procedure
should be called whenever an item with a layer tag is created or drawn.
The \fBset_layers\fR call does not have to be immediate.  For example, it
may be more efficient to create a set of items in a loop and then stack them
with a single call to \fBset_layers\fR after the loop.  \fBinsert_layer\fR
and \fBrm_layer\fR call \fBset_layers\fR implicitly.
.TP
\fInamespaceName\fB::xytolatlon\fR ?\fB-catch\fR? \fIx y ?x y ...?\fR
Returns the geographic coordinates corresponding to one or more pairs of
canvas coordinates in the canvas.  If the \fB-catch\fR option is present,
points for which the conversion is not defined are skipped.
Otherwise, an attempt to make an undefined conversion results in an error.
Return value is a list of form {{lat1 lon1} {lat2 lon2} ...}, in which each
{lat lon} corresponds to an "x y" pair in the input list.
.TP
\fInamespaceName\fB::latlontoxy\fR ?\fB-catch\fR? {\fIlat1 lon1\fR} ?{\fIlat2 lon2\fR} ...?
Returns a list of canvas coordinates in the canvas corresponding to one or more
pairs of geographic coordinates.  If the \fB-catch\fR option is present, points
for which the conversion is not defined are skipped.  Otherwise, an attempt to
make an undefined conversion results in an error.  Return value is a list of
form {x1 y1 x2 y2 ...} in which each "x y" pair corresponds to a {lat lon}
in the input list.
.TP
\fIpathName\fR \fBaddmenu\fR \fIname\fR
Adds a menu to \fIpathName\fR's menu bar.  This method will create a menu
button named \fIname\fR and associated menu.  Return value is the path name
of the associated menu, which user can then modify as described in the
\fBmenu\fR (n) man page.
.TP
\fIdeletemenu\fR \fBdeletemenu\fR \fIname\fR
Deletes a menu created with the \fBaddmenu\fR method.  The menu button and
associated menu will be destroyed.
.TP
\fInamespaceName\fB::delete\fR
Deletes the namespace, canvas, and all associated data.  This procedure should
be used instead of \fBnamespace delete\fR or Tk's \fBdestroy\fR command to
delete a map created by the \fBgeomap::igeomap\fR command.
.SH OTHER PROCEDURES
.TP
\fBgeomap::set_motion_bindings\fR \fImodifier button\fR
This procedure applies bindings that allow the user to move a map with the
mouse.
The type of mouse motion required to move the map depends on the projection.
If the projection uses a reference point, like Lamber Conformal Conic,
double-clicking selects a new projection reference point and map
center.  If the projection uses a reference longitude, the map can be
moved by dragging.  When the mouse is released after dragging, the final
longitude at the center of the map becomes the projection reference longitude,
unless the \fB-lazy\fR option is set.
\fImodifier\fR specifies an additional key to press while moving the map with
the mouse.  It should be one of \fB""\fR, \fBAlt\fR, \fBControl\fR, or
\fBShift\fR.  A non-empty modifier can help prevent spurious map motion.
\fIbutton\fR specifies the mouse button to push while moving the map.
It should be \fB1\fR, \fB2\fR, or \fB3\fR.
.TP
\fBgeomap::get_motion_bindings\fR
This query procedure returns a list of bindings that move maps created with
the \fBgeomap\fR command.
.SH FILES
The \fBgeomap::igeomap\fR command is part of the \fBtkgeomap_procs\fR
package, which is defined in file \fBtkgeomap_procs.tcl\fR of the tkgeomap
installation.
.SH SEE ALSO
.br
tkgeomap (n)
.SH AUTHOR
Gordon Carrie, user0@tkgeomap.org
