.\" 
.\" Copyright (c) 2005 Gordon D. Carrie. All rights reserved.
.\" 
.\" Licensed under the Open Software License version 2.1
.\" 
.\" Please address questions and feedback to user0@tkgeomap.org
.\"
.\" @(#) $Id: wdgeomap.n,v 1.17 2006/03/29 19:28:49 tkgeomap Exp $
.\"
.so man.macros
.TH geomap::wdgeomap n 2 Tk "Tk Extensions tkgeomap"
.SH NAME
wdgeomap \- create and manipulate a geographic map with interactive menus and bindings.
.SH SYNOPSIS
\fBpackage require wdgeomap\fR ?\fB2\fR?
.br
\fBgeomap::wdgeomap::create\fR \fImapName\fR \fIpathName\fR ?\fIoptions\fR?
.SH DESCRIPTION
The \fBgeomap::wdgeomap\fR procedure creates a frame named \fIpathName\fR,
which contains a canvas with a geographic map and associated menus.
When the \fBgeomap::wdgeomap\fR procedure is called, \fIpathName\fR should not
exist, although its parent should.  The \fBwdgeomap\fR procedure packs the
contents of frame \fIpathName\fR, but it does not submit it to any geometry
manager.  The application that creates the wdgeomap should eventually make
the frame visible using Tk geometry management commands, such as \fBpack\fR,
\fBplace\fR, or \fBgrid\fR.  The \fImapName\fR argument identifies the
widgets and associated data in the procedures described below.
These procedures create and manage \fBgeomap_lnarr\fR and \fBgeomap_place\fR
items in the canvas. These items display information at geographic locations,
as described in the \fBtkgeomap\fR (n) man page.  The resulting items 
receive the same \fB-refpoint\fR, \fB-projection\fR, \fB-scale\fR, and
\fB-rotation\fR configuration options so that the items are on the same map
in the canvas, and they are updated automatically when the map's cartographic
properties change.
This is more convenient than configuring each canvas item separately.
There are also procedures that create and manage menus and bindings that make
the map interactive.
.sp
The \fBgeomap::wdgeomap::create\fR procedure accepts additional
\fIoption value\fR pairs which initialize the map and canvas configuration.
These \fIoption value\fR pairs are passed to
the '\fBgeomap::wdgeomap::configure\fR \fImapName\fR' procedure, described
below.
.SH DEFAULT MENUS
Unless configured otherwise, a wdgeomap widget will have the following menus
in its menu bar.  Menus can be managed with the \fBaddmenu\fR, \fBgetmenu\fR,
and \fBdeletemenu\fR procedures described below.
.TP
\fBFile\fR
Contains selections that affect the map or application as a whole.  Default
selections are:
.RS
.TP
\fBPostscript\fR
Exports the map to a postscript file.  A dialog box prompts the user for a
file name and color mode.  The dialog box is given a \fBwdgeomap_ps_dlg\fR
bindtag for subsequent access.
.TP
\fBQuit\fR
Exits the application.
.RE
.TP
\fBProjection\fR
Selects the cartographic projection.
.TP
\fBScale\fR
Selects the cartographic scale.
.TP
\fBTop\fR
Specifies which compass direction is up.
.SH COMMANDS
.TP
\fBgeomap::wdgeomap::map_canvas\fR \fImapName\fR
Returns the path of canvas in which the map is drawn.
.TP
\fBgeomap::wdgeomap::mbar\fR \fImapName\fR
Returns the path of menu bar.
.TP
\fBgeomap::wdgeomap::configure\fR \fImapName\fR \fIoption value ?option value ...?\fR
This procedure specifies the cartographic properties
of the map and configures associated items in the canvas.  If a given
\fIoption\fR is not in the list below, the procedure applies
it to the canvas configuration.  If \fIoption\fR is not a valid canvas
configuration option either, the procedure raises an error.  The following
options are recognized.
.RS
.TP
\fB-refpoint\fR \fI{lat lon}\fR
Sets the map center to point \fI{lat lon}\fR.  Default value is \fB{0.0 0.0}\fR.
.TP
\fB-projname\fR \fIname\fR
Specifies the type of map projection.  \fIname\fR should identify a projection
from the list returned by the \fBgeomap::projections\fR command.
Unlike the \fBgeomap::projection\fR command, this
option does not require any projection parameters.  If the projection requires
a reference point, such as Orthographic and Lambert Conformal Conic, the
lat-lon at the center of the map is used.  If the projection requires a
reference longitude, such as Mercator, the longitude at the center of the map is
used.  Projections that require a hemisphere specification, such as Polar
Stereographic, use whichever hemisphere contains the map center.
Use the \fBgeomap::proj_info\fR procedure in \fBtclgeomap_procs\fR package
to identify the type of reference, either \fBlongitude\fR or \fBpoint\fR, for
each projection.
The default value for the \fB-projname\fR option is \fBMercator\fR.
.TP
\fB-scale\fR \fIscale\fR
Specifies the cartographic scale of the map.  The value can be a
floating point number or a cartographic scale (e.g. 1:30000000).  Default value
is \fB1.0e-7\fR.
.TP
\fB-rotation\fR \fIrotation\fR
Specifies the map's orientation.  The \fIrotation\fR should be a value known
to the \fB-rotation\fR option described in the \fBtkgeomap\fR (n) man page.
Default value is \fB0.0\fR.
.TP
\fB-update\fR \fIscript\fR
Specifies a script to evaluate whenever the \fB-refpoint\fR,
\fB-projection\fR, \fB-scale\fR, or \fB-rotation\fR option is changed.  This
makes it possible to update labels and legends around the map, add or remove
items as the scale changes, and perform any other tasks that depend on the
map's cartographic characteristics.  \fIscript\fR is evaluated at global scope.
Default value is \fB{}\fR.
.TP
\fB-boundcirclecolor\fR \fIcolor\fR
Specifies the color of the bounding circle for hemispheric projections.
Default value is \fBBlack\fR.
.TP
\fB-lazy\fR \fIboolean\fR
Deletes certain bindings associated with the \fB-projname\fR option.  This
option should be set to true if the widget appears to be wasting time
modifying the map projection without changing the map appearance.
This often happens when the map is zoomed in most of the time.
Value should be a Boolean.  Default value is \fB0\fR (false).
.TP
\fB-colormenu\fR \fIboolean\fR
Determines whether the \fBFile\fR menu will have a \fBColors\fR selection.
This selection creates a dialog in which the user interactively selects
colors for map elements whose colors have been specified by the \fBsetcolor\fR
procedure described below.
The dialog box will have a button for each such map element.  When the user
presses one of these buttons, a color selection dialog will appear.  If the
user selects a color and clicks \fBOK\fR in the color selection dialog,
the application will update the elements color with a call to \fBsetcolor\fR.
The actual update tasks should be performed by a script assigned to the element
with a call to the \fBsetcolorscript\fR procedure.
The \fBSave\fR button saves color values currently in the dialog to a file
identified by the user.  
The \fBLoad\fR button retrieves values from a file created with the \fBSave\fR
button.  The \fBUndo\fR button retrieves previous color configurations.
If the \fB-colormenu\fR configuration option is set to true, the application
should use the \fBsetcolor\fR and \fBsetcolorscript\fR procedures to
manage all colors in the map.  This will allow the user to modify colors
conveniently in one place.  The color values and scripts that use them will
depend on what data the map displays and how it is imported.
When the the dialog box is created, a call to \fBbindtags\fR adds a tag named
\fBwdgeomap_color_dlg\fR to its toplevel.  This allows applications to add
bindings to it, such as context sensitive help.
.TP
\fB-layermenu\fR \fIboolean\fR
Determines whether the \fBFile\fR menu will have a \fBLayers\fR selection.
This selection creates a dialog in which the user can adjust the stacking order
of certain canvas items by dragging rectangles representing canvas
tags.  Items without tags listed in the dialog move to the bottom of the
stacking order.  Tags can be added to and removed from this dialog with the map
\fBadd_layer\fR and \fBrm_layer\fR procedures, described below.
When the the dialog box is created, a call to \fBbindtags\fR adds a tag named
\fBwdgeomap_layer_dlg\fR to its toplevel.  This allows applications to add
bindings to it, such as context sensitive help.
.TP
\fB-projections\fR \fIprojectionList\fR
Lists the projection names in the \fBProjections\fR menu.  Selecting a
projection from the menu sets the \fB-projname\fR option to that projection.
Default value is \fB{CylEqDist Mercator CylEqArea LambertConfConic
LambertEqArea Stereographic PolarStereographic Orthographic}\fR.  If value is
\fB{}\fR, the \fBProjections\fR menu is not displayed.
.TP
\fB-scales\fR \fIscaleList\fR
Lists the scales in the \fBScales\fR menu at the top of the
map.  Selecting a scale from the menu sets the \fB-scale\fR option to that
scale.  Default value is \fB{1:10000000 1:20000000 1:30000000 1:45000000
1:60000000 1:90000000 1:120000000}\fR.  If value is \fB{}\fR, the \fBScales\fR
menu is not displayed.
.TP
\fB-rotationmenu\fR \fIboolean\fR
Specifies whether or not the \fBTop\fR menu should be displayed.  Selecting
a direction from the menu sets the \fB-rotation\fR option so that the chosen
direction is at the top of the map.  Value should be a Boolean.  Default value
is \fB1\fR.
.RE
.TP
\fBgeomap::wdgeomap::cget\fR \fImapName\fR \fIoption\fR
Retrieves a value from the map configuration.  \fIoption\fR can be one of the
options known to the \fBconfigure\fR procedure, or a canvas option.
.TP
\fBgeomap::wdgeomap::draw\fR \fImapName\fR \fItype name\fR ?\fIoption value ...\fR?
This procedure creates or modifies a map item in the canvas.  \fItype\fR
identifies the item type, which must be either \fBgeomap_lnarr\fR or
\fBgeomap_place\fR.
\fIname\fR identifies the linearray or place displayed in the item.
It should be the name of a linearray or place command returned by the
\fBgeomap::lnarr\fR or \fBgeomap::place\fR command.  See the \fBtclgeomap\fR
(n) man page for more information on linearrays and places.
If the command for the linearray or place is in a non-global namespace,
\fIname\fR MUST BE FULLY QUALIFIED.
If \fItype\fR and \fIname\fR refer to an already existing item, the item is
modified.  Otherwise, a new item is created.
Item options that define the geographic map, specifically \fB-refpoint\fR,
\fB-projection\fR, \fB-rotation\fR, and \fB-scale\fR, will be obtained from
the map object.
Other options that affect the item's appearance can be given in the
\fIoption value\fR list.  See \fBtkgeomap\fR (n) man page for discussion of
recognized options.  Any options in the \fIoption value\fR list that define
the geographic map will be ignored.
In addition to any tags specified in the option list, the item will have the
following tags: \fIname\fR, and \fBgeomap\fR.
These tags help identify the item in other procedures and configuration tasks.
If the map manages layers, the application should call \fBset_layers\fR
after drawing a new line or place.  See below for discussion of layer
management.
This procedure returns the item identifier for the new or modified item.
.TP
\fBgeomap::wdgeomap::erase\fR \fImapName\fR \fItype name\fR
Deletes an item created by the \fBdraw\fR procedure.  \fItype\fR and \fIname\fR
should be as given to the \fBdraw\fR procedure.
.TP
\fBgeomap::wdgeomap::setcolor\fR \fImapName\fR \fIelement\fR \fIcolor\fR
Associates a color with a map element.  The \fIelement\fR should identify
map items that share a color, e.g. \fIwater\fR, \fIland\fR, \fIplaces\fR, etc.
The \fIcolor\fR should be intelligible to \fBTk_GetColor\fR.
After the color for the element is set, a script specified with a call to
\fBsetcolorscript\fR is invoked.  This script should draw or configure the
affected map elements using \fBcolor\fR.
.TP
\fBgeomap::wdgeomap::setcolorscript\fR \fImapName\fR \fIelement script\fR
Specifies a script to run when the color associated with \fIelement\fR is
changed with a call to \fIsetcolor\fR.
.TP
\fBgeomap::wdgeomap::getcolor\fR \fImapName\fR \fIelement\fR
Returns the color value specified in the last call to \fBsetcolor\fR for
element, or \fB""\fR if their is no color for the element.
.TP
\fBgeomap::wdgeomap::add_layer\fR \fImapName\fR \fIlayer\fR
Adds a layer to the map layer hierarchy, if the layer is not already present.
If \fIlayer\fR is already in the hierarchy, due to an earlier call to
\fBadd_layer\fR, this procedure does nothing.  Once \fIlayer\fR has been
added, the \fBFile->Layers\fR dialog can manipulate the stacking order of map
canvas items with tag \fIlayer\fR.  Layers can be arranged in a hierarchy.
A list element in \fIlayer\fR denotes a separate level in the map hierarchy.
The element before a list element identifies a parent.  Subsequent elements
denote a child or hierarchy of children.  If several hierarchical levels of
layers are present, the dialog will show them in a tree diagram.  Each path
through the tree is one layer.  All children of an item move with the item.
An item can only be moved relative to its siblings (other children of its
parent).  An application should call \fBset_layers\fR after calling
\fBadd_layer\fR and before the display is updated for the new stacking order
to take effect in the display.
.TP
\fBgeomap::wdgeomap::rm_layer\fR \fImapName\fR \fIlayer\fR
Removes tag \fIlayer\fR from the list of layers managed by the
\fBFile->Layers\fR dialog.  Canvas items with tag \fIlayer\fR will still exist,
but the \fBFile->Layers\fR dialog will no longer control their stacking order.
An application should call \fBset_layers\fR after calling \fBrm_layer\fR
and before the display is updated for the new stacking order to take effect
in the display.
.TP
\fBgeomap::wdgeomap::set_layers\fR \fImapName\fR
Puts items with layer related tags into their stacking order.  The
\fBset_layers\fR call does not have to be immediate.  For example, it
may be more efficient to create a set of items in a loop and then stack
them with a single call to \fBset_layers\fR after the loop.  Applications
should call \fBset_layers\fR after creating canvas items with the canvas
\fBcreate\fR subcommand, drawing geomap_lnarr or geomap_place items with
the wdgeomap \fBdraw\fR subcommand, or modifying the layer sequence with the
wdgeomap \fBadd_layer\fR or \fBrm_layer\fR subcommands.
.TP
\fBgeomap::wdgeomap::xytolatlon\fR \fImapName\fR ?\fB-catch\fR? \fIx y ?x y ...?\fR
Returns the geographic coordinates corresponding to one or more pairs of
canvas coordinates in the canvas.  If the \fB-catch\fR option is present,
points for which the conversion is not defined are skipped.
Otherwise, an attempt to make an undefined conversion results in an error.
Return value is a list of form {{lat1 lon1} {lat2 lon2} ...}, in which each
{lat lon} corresponds to an "x y" pair in the input list.
.TP
\fBgeomap::wdgeomap::latlontoxy\fR \fImapName\fR ?\fB-catch\fR? {\fIlat1 lon1\fR} ?{\fIlat2 lon2\fR} ...?
Returns a list of canvas coordinates in the canvas corresponding to one or more
pairs of geographic coordinates.  If the \fB-catch\fR option is present, points
for which the conversion is not defined are skipped.  Otherwise, an attempt to
make an undefined conversion results in an error.  Return value is a list of
form {x1 y1 x2 y2 ...} in which each "x y" pair corresponds to a {lat lon}
in the input list.
.TP
\fBgeomap::wdgeomap::addmenu\fR \fImapName\fR \fIname\fR
Adds a menu to \fImapName\fR's menu bar.  This method will create a menu
button named \fIname\fR and associated menu.  Return value is the path name
of the associated menu, which user can then modify as described in the
\fBmenu\fR (n) man page.
.TP
\fBgeomap::wdgeomap::getmenu\fR \fImapName\fR \fIname\fR
Retrieves the path name of the menu button created by \fBaddmenu\fR for
\fIname\fR.
.TP
\fBgeomap::wdgeomap::deletemenu\fR \fImapName\fR \fIname\fR
Deletes a menu created with the \fBaddmenu\fR method.  The menu button and
associated menu will be destroyed.
.TP
\fBgeomap::wdgeomap::delete\fR \fImapName\fR
Deletes the map widget, and all associated data.
.TP
\fBgeomap::wdgeomap::set_motion_bindings\fR \fImodifier button\fR
This procedure applies bindings that allow the user to move a map with the
mouse.
The type of mouse motion required to move the map depends on the projection.
If the projection uses a reference point, like Lamber Conformal Conic,
double-clicking selects a new projection reference point and map
center.  If the projection uses a reference longitude, the map can be
moved by dragging.  When the mouse is released after dragging, the final
longitude at the center of the map becomes the projection reference longitude,
unless the \fB-lazy\fR option is set.
\fImodifier\fR specifies an additional key to press while moving the map with
the mouse.  It should be one of \fB""\fR, \fBAlt\fR, \fBControl\fR, or
\fBShift\fR.  A non-empty modifier can help prevent spurious map motion.
\fIbutton\fR specifies the mouse button to push while moving the map.
It should be \fB1\fR, \fB2\fR, or \fB3\fR.
.TP
\fBgeomap::wdgeomap::get_motion_bindings\fR
This query procedure returns a list of bindings that move map items created
with the \fBdraw\fR procedure.
.SH FILES
\fBwdgeomap.tcl\fR
.SH SEE ALSO
tclgeomap (n)
.br
tkgeomap (n)
.SH AUTHOR
Gordon Carrie, user0@tkgeomap.org
