#--------------------------------------------------------------------
# TCLGEOMAP_LIBRARY --
#
#	Locate the tclgeomap library.
#
# Arguments:
#	none
#
# Results:
#	Defines the following var:
#		TCLGEOMAP_LIB_SPEC	Command line options to CC to link in
#					tclgeomap library.
#		
#--------------------------------------------------------------------

AC_DEFUN(TCLGEOMAP_LIBRARY, [
    if test "${SHARED_BUILD}" = "1"
    then
	AC_MSG_CHECKING(for tclgeomap)
	cat > find_tclgeomap.tcl << END
	lappend auto_path ${prefix}/lib ${TCLGEOMAP_LIB}
	package require tclgeomap ${PACKAGE_VERSION}
	puts \$::geomap::tclgeomap_lib_file
END
	TCLGEOMAP_BIN_FILE=`${TCLSH_PROG} ./find_tclgeomap.tcl 2> /dev/null`
	rm -f find_tclgeomap.tcl
	if test x"$TCLGEOMAP_BIN_FILE" = "x"
	then
	    AC_MSG_ERROR(shared build specified but could not find shared tclgeomap library)
	else
	    AC_MSG_RESULT(tclgeomap library is $TCLGEOMAP_BIN_FILE)
	fi

	# Define tclgeomap library specifier for link command,
	# if system needs it.

	tclgeomap_lib_dir=`dirname ${TCLGEOMAP_BIN_FILE}`
	tclgeomap_lib_spec="-L$tclgeomap_lib_dir -ltclgeomap${PACKAGE_VERSION}"
	system=`uname -s`-`uname -r`
	if test "$?" -ne 0 ; then
	    AC_MSG_RESULT([unknown (can't find uname command)])
	    system=unknown
	fi
	TCLGEOMAP_LIB_SPEC=""
	case $system in
	    Darwin-*)
		TCLGEOMAP_LIB_SPEC=$tclgeomap_lib_spec
		;;
	    SunOS-5*)
		TCLGEOMAP_LIB_SPEC=$tclgeomap_lib_spec
		;;
	esac
	AC_SUBST(TCLGEOMAP_LIB_SPEC)
    fi
])
#--------------------------------------------------------------------
# TCLGEOMAP_HEADERS --
#
#	Locate the tclgeomap library.
#
# Arguments:
#	none
#
# Results:
#	Defines the following var:
#		TCLGEOMAP_INCLUDES	fully path name of directory with
#					tclgeomap.h
#		
#--------------------------------------------------------------------

AC_DEFUN(TCLGEOMAP_HEADERS, [
    AC_MSG_CHECKING(for Tclgeomap public headers)
    AC_ARG_WITH(tclgeomap-include, [ --with-tclgeomap-include      directory containing the public tclgeomap header files.], with_tclgeomap_include=${withval})
    if test x"${with_tclgeomap_include}" != x ; then
	if test -f "${with_tclgeomap_include}/tclgeomap.h" ; then
	    ac_cv_c_tclgeomap_h=${with_tclgeomap_include}
	else
	    AC_MSG_ERROR([${with_tclgeomap_include} directory does not contain Tclgeomap public header file tclgeomap.h])
	fi
    else
	AC_CACHE_VAL(ac_cv_c_tclgeomap_h, [
	    # Use the value from --with-tclgeomap-include, if it was given

	    if test x"${with_tclgeomap_include}" != x ; then
		ac_cv_c_tclgeomap_h=${with_tclgeomap_include}
	    else
		# Check in the includedir, if --prefix was specified
		eval "temp_includedir=${includedir}"
		for i in \
			`ls -d ${temp_includedir} 2>/dev/null` \
			${prefix}/include/tclgeomap${PACKAGE_VERSION} \
			/usr/local/include/tclgeomap${PACKAGE_VERSION} \
			/usr/include/tclgeomap${PACKAGE_VERSION} \
			/usr/local/include /usr/include ; do
		    if test -f "$i/tclgeomap.h" ; then
			ac_cv_c_tclgeomap_h=$i
			break
		    fi
		done
	    fi
	])
    fi
    if test x"${ac_cv_c_tclgeomap_h}" = x ; then
	AC_MSG_ERROR(tclgeomap.h not found.  Please specify its location with --with-tclgeomap-include)
    else
	AC_MSG_RESULT(${ac_cv_c_tclgeomap_h})
    fi
    INCLUDE_DIR_NATIVE=`${CYGPATH} ${ac_cv_c_tclgeomap_h}`
    TCLGEOMAP_INCLUDES=-I\"${INCLUDE_DIR_NATIVE}\"
    AC_SUBST(TCLGEOMAP_INCLUDES)
])
